/**
 * Package containing the model/simulation code for CSci 1902 Homework 3.
 */
package csci1902.hw3;

import java.util.*;
import java.awt.Point;

/**
 * Base class for simulation entities. All entities in the simulation must
 * inherit from this class.
 * 
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 * 
 */
public class Entity {
    private Point position;
    private Point nextPosition;
    private boolean alive = true;
    private boolean nextAlive = true;
    private World world;
    
    /**
     * Create a new entity
     * @param w The world associated with this entity
     * @param initPos The initial position of the entity
     */
    protected Entity(World w, Point initPos) {
        world = w;
        position = initPos;
    }
    
    /**
     * Retrieve the active configuration for this entity.
     * @return The configuration associated with this entity.
     */
    public World getWorld() {
        return world;
    }
    
    /**
     * Retrieve the position of the entity
     */
    public Point getPosition() {
        return position;
    }
    
    /**
     * Explicitly and immediately set the position of the entity. This method
     * should not be used in the simulation -- use
     * {@link #setNextPosition(Point)} instead.
     * 
     * @param pos The new position of the entity
     */
    public void setPosition(Point pos) {
        position = pos;
    }
    /**
     * Set the position the object will have in the next state.
     * @param p The next state position.
     */
    protected void setNextPosition(Point p) {
        nextPosition = p;
    }
    
    /**
     * Make the entity move in a random direction.
     */
    protected void moveRandomly() {
        List<Point> positions = getWorld().getNeighboringCells(getPosition());
        int l = positions.size();
        int i = (int) (l * getWorld().getRandomGenerator().getNumber()) % l;
        setNextPosition(positions.get(i));
    }
    
    /**
     * Query whether the entity is "alive" (active).
     * @return true if the entity is currently alive.
     */
    public boolean isAlive() {
        return alive;
    }
    /**
     * Query whether the entity is scheduled to die next state.
     * @return true if the entity will die when the next state is committed.
     */
    public boolean isDying() {
        return !nextAlive;
    }
    /**
     * Schedule the entity to die when the next state is committed.
     */
    protected void die() {
        nextAlive = false;
    }
    
    /**
     * Prepares the next state of the object.  The default implementation does
     * nothing (appropriate for non-mobile, do-nothing entities).  This method
     * should not actually make changes, but rather set up changes to be made
     * in commitNextState.
     */
    public void act(List<Entity> neighbors) {
    }

    /**
     * Commit the next state changes.  This makes the changes made in
     * prepareNextState effective.
     */
    public void update() {
        if (nextPosition != null) {
            position = nextPosition;
            nextPosition = null;
        }
        alive = nextAlive;
    }
    
    /**
     * Determine if the entity is a food source.
     * @return true if the entity is a food source.
     */
    public boolean isFoodSource() {
        return false;
    }
    /**
     * Collect food from the entity.  This method is only valid on food sources.
     * @param amount The amount of food to collect.
     * @return The amount of food actually collected.
     * @throws UnsupportedOperationException if the entity is not a food source.
     */
    public int collectFood(int amount) {
        throw new UnsupportedOperationException("Not a food source");
    }
    
    /**
     * Determine if the entity is a food sink.
     * @return true if the entity can accept food deposits.
     */
    public boolean isFoodSink() {
        return false;
    }
    /**
     * Deposit food at the entity.  This method is only valid on food sinks.
     * @param amount The amount of food to deposit.
     * @throws UnsupportedOperationException if the entity is not a food sink.
     */
    public void depositFood(int amount) {
        throw new UnsupportedOperationException("Not a food sink");
    }
    
    /**
     * Query whether the entity is a waste source (can have waste collected).
     * @return true if the entity can have waste collected from it
     */
    public boolean isWasteSource() {
        return false;
    }
    
    /**
     * Collect waste from a waste source.  Only valid if {@link #isWasteSource}
     * returns {@code true}.
     * @param waste The amount of waste to collect
     * @return The amount of waste actually collected
     */
    public int collectWaste(int waste) {
        throw new UnsupportedOperationException("Not a waste source");
    }
    
    /**
     * Query whether the entity is a waste sink (accepts waste deposits)
     * @return true if the entity accepts waste deposits.
     */
    public boolean isWasteSink() {
        return false;
    }
    
    /**
     * Deposit waste in a waste sink.  Only valid if {@link #isWasteSink}
     * return true.
     * @param waste The amount of waste to deposit
     */
    public void depositWaste(int waste) {
        throw new UnsupportedOperationException("Not a waste sink");
    }
    
    /**
     * Determine whether the entity is edible to spiders.
     * @return true if the entity is edible.
     */
    public boolean isEdible() {
        return false;
    }
    /**
     * Attempt to eat the entity.
     * @return {@code true} if the entity was successfully eaten. 
     */
    public boolean eat() {
        throw new UnsupportedOperationException("Inedible entity");
    }
    
    /**
     * Check for equality.  Two entities are equal if and only if they are the
     * same object.
     */
    public boolean equals(Object o) {
        return o == this;
    }
}
